// ==============================================================
// Vitis HLS - High-Level Synthesis from C, C++ and OpenCL v2020.2 (64-bit)
// Copyright 1986-2020 Xilinx, Inc. All Rights Reserved.
// ==============================================================

`timescale 1 ns / 1 ps

module Delay_q_Array_core (
    clk,
    ce,
    reset,
    din,
    addr,
    dout);

parameter DATA_WIDTH = 32'd32;
parameter ADDR_WIDTH = 32'd1;
parameter DEPTH = 2'd1;

input clk;
input ce;
input reset;
input [DATA_WIDTH-1:0] din;
input [ADDR_WIDTH-1:0] addr;
output [DATA_WIDTH-1:0] dout;

reg [ADDR_WIDTH-1:0] counter = {ADDR_WIDTH{1'b0}};
reg reset_sel;

reg[DATA_WIDTH-1:0] ShiftRegMem[0:DEPTH-1];
integer i;

initial
begin
    for(i=0;i<DEPTH;i=i+1)
        ShiftRegMem[i] <= {DATA_WIDTH{1'b0}};
end

always @ (posedge clk)
begin
    if (ce)
    begin
        for(i=0;i<DEPTH-1;i=i+1)
            ShiftRegMem[i+1] <= ShiftRegMem[i];
        ShiftRegMem[0] <= din;
    end
end

always @ (posedge clk)
begin
    if (reset)
        counter <= {ADDR_WIDTH{1'b0}};
    else if (ce && counter < DEPTH - 2'd1)
        counter <= counter + 1'b1;
    else
        counter <= counter;
end

always @ (counter or addr)
begin
    if (counter < addr)
        reset_sel = 1'b1;
    else
        reset_sel = 1'b0;
end

assign dout = reset_sel ? {DATA_WIDTH{1'b0}} : ShiftRegMem[addr];

endmodule

module Delay_q_Array (
    clk,
    reset,
    address0,
    ce0,
    we0,
    d0,
    q0);

parameter DataWidth = 32'd32;
parameter AddressRange = 32'd1;
parameter AddressWidth = 32'd1;

input clk;
input reset;
input [AddressWidth-1:0] address0;
input ce0;
input we0;
input [DataWidth-1:0] d0;
output [DataWidth-1:0] q0;

Delay_q_Array_core #(
    .DATA_WIDTH( DataWidth ),
    .ADDR_WIDTH( AddressWidth ),
    .DEPTH( AddressRange ))
Delay_q_Array_core_U(
    .clk(clk),
    .ce(we0),
    .reset(reset),
    .din(d0),
    .addr(address0),
    .dout(q0)
);

endmodule
